#Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#SPDX-License-Identifier: MIT-0
#
#Permission is hereby granted, free of charge, to any person obtaining a copy of this
#software and associated documentation files (the "Software"), to deal in the Software
#without restriction, including without limitation the rights to use, copy, modify,
#merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#permit persons to whom the Software is furnished to do so.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import boto3
import logging
import os
import json

LOGGER = logging.getLogger()
LOGGER.setLevel(os.getenv('LOGLEVEL', logging.INFO))

session = boto3.Session()

def assume_role(aws_account_number, role_name):
    """
    Assumes the provided role in each account and returns an EC2 client
    :param aws_account_number: AWS Account Number
    :param role_name: Role to assume in the target account
    :return: EC2 client in the specified AWS Account
    """
    try:
        sts_client = boto3.client('sts')
        partition = sts_client.get_caller_identity()['Arn'].split(":")[1]
        response = sts_client.assume_role(
            RoleArn=f'arn:{partition}:iam::{aws_account_number}:role/{role_name}',
            RoleSessionName='IncreaseQuotaLimit'
        )
        sts_session = boto3.Session(
            aws_access_key_id=response['Credentials']['AccessKeyId'],
            aws_secret_access_key=response['Credentials']['SecretAccessKey'],
            aws_session_token=response['Credentials']['SessionToken']
        )
        LOGGER.info(f'Assumed session for {aws_account_number}')

        return sts_session
    except Exception as e:
        LOGGER.error(f'Unable to assume specified role in account: {e}')


def lambda_handler(event, context):
    LOGGER.info(f'Event: {event}')

    try:
        message = json.loads(event['Records'][0]['Sns']['Message'])
        LOGGER.info(f'Message: {message}')
        ROLE = os.environ['RoleName']
        SERVICE_CODE = os.environ['ServiceCode']
        QUOTAS_CODE = os.environ['QuotasCode']
        DESIRED_LIMIT = os.environ['DesiredLimit']
        TARGET_ACCOUNT = message['detail']['serviceEventDetails']['createManagedAccountStatus']['account']['accountId']
        REGION_LIST = os.environ['Regions'].split(',')
    except Exception as e:
        LOGGER.exception(f'Error parsing event and variables: {e}')

    try:
        target_session = assume_role(aws_account_number=TARGET_ACCOUNT, role_name=ROLE)
        for region in REGION_LIST:
            #create the client for the target account
            quotas_client = target_session.client(
                'service-quotas',
                region_name=region
            )

            #request the increase using the codes
            response = quotas_client.request_service_quota_increase(
                ServiceCode=SERVICE_CODE,
                QuotaCode=QUOTAS_CODE,
                DesiredValue=int(DESIRED_LIMIT)
            )

            if response['ResponseMetadata']['HTTPStatusCode'] == 200:
                LOGGER.info(f'Successfully requested service limit increase for AWS account {TARGET_ACCOUNT} in region {region}')
            else:
                LOGGER.error(f'Failed to successfully request a service limit increase for AWS account {TARGET_ACCOUNT} in region {region}')
    except Exception as e:
        LOGGER.error(f'Error requesting service limit increase: {e}')